                          program specx
c---------------------------------------------------------------------
c     ------------------------------------
c     --- KKR-CPA + spin-orbit version ---
c     ------------------------------------
c      KKR-CPA band structure calculation
c      originaly coded by H.Akai and M.Akai, 1980, on ACOS
c           adapted to NEC sx1&2, Aug. 1986 at Osaka
c           adapted to CRAY xmp, Aug. 1986 at Juelich
c           adapted to HITAC s820, Nov. 1989 at ISSP
c           adapted to HP 730, Julv. 1992
c                latest version on 11 Feb. 1996
c      CPA is implemented by H.Akai, 7 Sep. 1996, Osaka
c      Adapted to spin-orbit included version, 18 April, 1997, Osaka
c      Fixed spin moment procedure implemented, 8 Aug. 1999, Duisburg
c---------------------------------------------------------------------
      implicit real*8 (a-h,o-z)
      parameter
     & (natmmx=6, ncmpmx=6, msizmx=96, mxlmx=4, nk1x=1250, nk3x=300,
     & msex=201, ngmx=15, nrpmx=650, ngpmx=650, npmx=350, msr=400)
c
c--- the following part is used for all cases.
      parameter
     & (lengx=msex, ndmx=natmmx*(natmmx-1)+1, mxlfac=5-mxlmx,
     & lastmx=2900d0/3d0**mxlfac, ntypmx=ncmpmx, nkmx=nk1x+nk3x)
c
c
c     note: mxlmx is l_max + 1 where l_max is the maximum angular
c           momentum used in the calculation.
c           Normally, the maximum size needed for the work area
c           is determined by the maximum of 4*natmmx+ngpmx+ntypmx,
c           ,ncmpx*msr,5*msr, 2*mxlmx**2*ncmpmx*msex, and 7620.
c           The last number, 7620, depends on the number of
c           k-points used in the BZ; 7620 corresponds to the
c           case where 32*10^4 k-points in the full BZ are used.
c           This number, however, should not be reduced to less
c           than 7128 even if less k-points in the BZ are used.
c           lastmx should be larger thatn 2, 37, 243, 964, and
c           2854 for mxl=1, 2, 3, 4, and 5,respectively.
c
c
      complex*16 e(msex,2),tch((2*mxlmx-1)**2*ngmx*nkmx*ndmx)
     &          ,pexf(npmx*nkmx*ndmx),wt(ngmx,3,msex,2)
     &          ,wkc(1),detl(msex,2),gfree(msex)
     &          ,ff(mxlmx**4*natmmx*msex)
     &          ,tmt(msex*mxlmx**2*ncmpmx),phf(msex*mxlmx**4*ncmpmx)
     &          ,str(msex*mxlmx**2*ncmpmx),tc(msex*ngmx)
     &          ,ess(msex*(2*mxlmx-1)**2)
     &          ,tcpa(msex*mxlmx**4*natmmx*2)
     &          ,phase(msex*mxlmx**2*ncmpmx)
     &          ,ck(msex*mxlmx**4*natmmx),cj(msex*mxlmx**4*natmmx)
     &          ,urotat((2*mxlmx-1)**2,mxlmx,24),spctrl(msex*nk3x)
     &          ,uu(2*(2*mxlmx-1)**2)
c
      real*8     anclr(ncmpmx),rmt(ntypmx),ancp(ncmpmx,2)
     &          ,field(ntypmx),atmicp(3,natmmx)
     &          ,elvl(ngmx),tm(ngmx,ngmx),clks(lastmx),zdmy(ncmpmx)
     &          ,corlvl(18,ncmpmx,2),dr(msr,ncmpmx),xr(msr,ncmpmx)
     &          ,vkp(3,nkmx),rpt(3,nrpmx),gpt(3,ngpmx),wtkp(nkmx)
     &          ,amdlng(natmmx,natmmx),prr(npmx,nkmx),rms(ncmpmx,2)
     &          ,hh(npmx,(2*mxlmx-1)**2,nkmx),v1(msr,ncmpmx,2)
     &          ,v2(msr,ncmpmx,2),v3(msr,ncmpmx,2)
c
      real*8     wk(1),ro(msr,ncmpmx,2),rorg(20,ncmpmx,2)
     &          ,tchc(ngmx,mxlmx**2,ncmpmx),tchs(ngmx,mxlmx**2,ncmpmx)
     &          ,fcs(3,mxlmx**2,ncmpmx)
     &          ,rstr(msr*mxlmx**2*ngmx,ncmpmx,2)
     &          ,dosef(mxlmx**2,ncmpmx,2),esic(ncmpmx,2)
     &          ,sm(ngmx,mxlmx**2,ncmpmx,2),anc(ncmpmx)
     &          ,tof(mxlmx**2,ncmpmx,2),f(mxlmx**2,ncmpmx,ngmx,2)
     &          ,rmtd(natmmx),q(ncmpmx),cm(ngmx,mxlmx**2,ncmpmx)
     &          ,config(18,ncmpmx,2),conc(ncmpmx),hhf(ncmpmx)
     &          ,xmd(ncmpmx,msex,2,2),r(3,3)
      real*8     wk1(4*natmmx+ngpmx+ntypmx),wk2(ncmpmx*msr)
     &          ,wk3(7*msr),wk4(2*mxlmx**2*ncmpmx*msex),wk5(7620)
     &          ,wk6((2*mxlmx+2)*msr)
     &          ,wk7((2*mxlmx-1)**2*mxlmx*msex)     
     &          ,wk8(2*(2*mxlmx-1)**2*mxlmx*24)     
c
      integer    itype(natmmx),ncub(lastmx),iwk(1)
     &          ,iblk(natmmx,natmmx),iwtyp(ncmpmx),lmxblk(ndmx)
     &          ,match(18,ncmpmx,2)
     &          ,ncmp(ntypmx),korder(nkmx),irotat(natmmx*48)
     &          ,iatm(ntypmx),lmxtyp(ntypmx),mxlcmp(ncmpmx)
     &          ,iatmp(natmmx),itblk(5,natmmx**2)
c
      character  go*3,file*36,brvtyp*3,reltyp*6,sdftyp*12,magtyp*4
     &          ,outtyp*6,type(ntypmx)*8,atmtyp(natmmx)*8,title*400
     &          ,bzqlty*8,record*4,atmicv(3,natmmx)*24,cwtyp*4
      logical    convrg(msex)
c
c     common     tch,pexf,rstr,hh,wkc
c
      equivalence (wkc,wk,iwk,wk1,wk2,wk3,wk4,wk5,wk6,wk7,wk8)
c
c     data ef0/7d-1/, dex/3d-3/, emxr/1d0/, mse0/65/
      data ef0/7d-1/, dex/3d-3/, emxr/1d0/, mse0/35/
     &    ,xlim/1.5d0/, ng/ngmx/, meshr/msr/, tol/1d-6/
     &    ,ids/0/, inv/0/
c     ----------------------------------------------------
      call uclock(cpu)
      call setomp
      nwk=max(4*natmmx+ngpmx+ntypmx,ncmpmx*msr
     &       ,5*msr,(2*mxlmx+2)*msr,2*mxlmx**2*ncmpmx*msex
     &       ,(12+ncmpmx)*msr,2*(2*mxlmx-1)**2*mxlmx*24,7620)
      niwk=2*nwk
c     write(*,*)niwk
      do 10 icmp=1,1000
      call readin(go,file,brvtyp,a,coa,boa,alpha,beta,gamma
     &           ,edelt,ewidth,reltyp,sdftyp,magtyp,record,outtyp
     &           ,bzqlty,maxitr,pmix,ntyp,type,ncmp,rmt,field,lmxtyp
     &           ,anclr,conc,natm,atmicv,atmtyp,r,*20)
c
      if(go .eq. 'go' .or. go .eq. 'dsp' .or. go .eq. 'dos' .or. go
     &   .eq. 'log' .or. go .eq. 'mcd' .or. go .eq. 'xmd' .or.
     &   go .eq. 'spc' .or. go .eq. 'fsm') then
      ncmpx=0
      ncmax=0
      mxl=1
      do 30 i=1,ntyp
      mxl=max(mxl,lmxtyp(i)+1)
      ncmax=max(ncmax,ncmp(i))
   30 ncmpx=ncmpx+ncmp(i)
      if(mxl .gt. mxlmx) then
      call errtrp(3,'specx','lmx may be too large, check it.')
      endif
      if(ncmpx .gt. ncmpmx) then
      call errtrp(3,'specx','ncmpx too large')
      go to 10
      endif
      if(natm .gt. natmmx) then
      call errtrp(3,'specx','natm too large')
      go to 10
      endif
      ids=0
      if(go .eq. 'dsp') then
c      --- display the result without any iteration process.
c     record='2nd'
      outtyp='quit'
      ids=2
      maxitr=1
      mse=min(mse0,msex)
      else if(go .eq. 'dos') then
c     --- in addition, DOS will be displayed.
c     record='2nd'
      outtyp='quit'
      ids=1
      maxitr=1
c     --- also the number of the energy mesh is set to be the maximum value.
      mse=msex
c     mse=5
c     ewidth=0.1
      else if(go .eq. 'mcd' .or. go .eq. 'xmd') then
c     --- in addition, MCD will be displayed.
c     record='2nd'
      outtyp='quit'
      ids=3
      maxitr=1
c     --- also the number of the energy mesh is set the maximum value.
      mse=msex
      else if(go .eq. 'spc') then
c     --- Bloch spectral functions are output
c      --- display the result without any iteration process.
c     record='2nd'
      outtyp='quit'
c     bzqlty='0'
      ids=4
      maxitr=1
      mse=msex
      else if(go .eq. 'fsm') then
c     --- fixed spin moment procedure used
      ids=5
      mse=min(mse0,msex)
      else
      mse=min(mse0,msex)
      endif
c     --- generate table that returns (j,i)->jip or jip->(j,i) maps.
c         the mapping is established by calling later a subroutine
c         entry jip as 'call jip(i,j,ji)' or 'call jipinv(i,j,ji)',
      call inijip(ncmp,ntyp,ncmax)
c     --- normalize the concentration of the component atoms.
c     --- also mxlcmp is set.
      ji1=0
      ji2=0
      do 40 i=1,ntyp
      cnorm=0d0
      do 50 j=1,ncmp(i)
      ji1=ji1+1
      mxlcmp(ji1)=lmxtyp(i)+1
   50 cnorm=cnorm+conc(ji1)
      if(abs(cnorm) .lt. 1d-30) call errtrp(1,'specx','illegal conc')
      do 40 j=1,ncmp(i)
      ji2=ji2+1
   40 conc(ji2)=conc(ji2)/cnorm
      call ty2ity(atmtyp,natm,type,ntyp,itype)
      msiz=0
      do 60 i=1,natm
   60 msiz=msiz+(lmxtyp(itype(i))+1)**2
      if(msiz .gt. msizmx) call errtrp(1,'specx','msizmx too small')
c
c     --- number of available cores of the system is obtained
c
      call sbtime(0,0)
      call spmain
     & (e,tch,pexf,wt,wkc,detl,ff,tmt,phf,str,tc
     & ,anclr,rmt,ancp,field,atmicv,atmicp,elvl,tm,clks,zdmy
     & ,corlvl,dr,xr,vkp,rpt,gpt,amdlng,prr,rms,hh,v1,v2,v3
     & ,wk,ro,rorg,cm,tchc,tchs,fcs,rstr,dosef,esic
     & ,sm,anc,tof,f,rmtd,q,wtkp,hhf,match,itype,ncub,iwk
     & ,iblk,itblk,iwtyp,config,go,file,brvtyp,reltyp
     & ,sdftyp,magtyp,outtyp,type,atmtyp,cwtyp,title,bzqlty
     & ,record,ef0,dex,emxr,xlim,conc,mse,ng,mxl,tol,ids,inv
     & ,a,coa,boa,alpha,beta,gamma,edelt,ewidth,maxitr,pmix,xmd
     & ,ntyp,natm,meshr,ndmx,npmx,lastmx,nrpmx,ngpmx,nkmx,nwk
     & ,niwk,ncmp,ncmpx,gfree,ess,tcpa,phase,korder
     & ,convrg,urotat,uu,irotat,ck,cj,iatm,spctrl,nk3x,lmxtyp
     & ,mxlcmp,msiz,lmxblk,iatmp,r)
c
      call sbtime(-1,0)
      endif
   10 continue
   20 call uclock(cpu)
      write(*,'(1x,a,t15,f12.2,a)')'cpu used ',cpu,' sec'
      call endomp
      end
